class UserIsNoneObject(Exception):
    ...


from .channel import *
from .guild import *
from httpc import *


class User:

    """
    #### The User object contains data
    #### about the connected object by token.
    """

    def __init__(self, user: dict, token: str) -> None:
        self.user: dict = user
        self.token: str = token

    def __check__(self) -> None:
        if self.user is None:
            raise UserIsNoneObject("The user parameter was not passed...")

    @property
    def id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: id user`
        """

        self.__check__()

        return int(self.user["id"])

    @property
    def username(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: username`
        """

        self.__check__()

        return str(self.user["username"])

    @property
    def global_name(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: global name`
        """

        self.__check__()

        return str(self.user["global_name"])

    @property
    def avatar(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: string hash`
        """

        self.__check__()

        return str(self.user["avatar"])

    def get_guilds(self) -> list[Guild]:
        """
        Params:
            `None`

        Returns:
            `list[Guild]: list guilds current user`
        """

        self.__check__()

        connect_request = MentordRequest(
            url="https://discordapp.com/api/v9/users/@me/guilds",
            headers={"Authorization": self.token, "Content-Type": "application/json"},
        ).GET()

        if connect_request.status_code != 200:
            raise RequestError(
                "Error request, status code: {0}".format(connect_request.status_code)
            )

        return list[Guild](Guild(guild) for guild in connect_request.json())

    def get_channels(self) -> list[Channel]:
        """
        Params:
            `None`

        Returns:
            `list[Channel]: list channels current user`
        """

        self.__check__()

        connect_request = MentordRequest(
            url="https://discordapp.com/api/v9/users/@me/channels",
            headers={"Authorization": self.token, "Content-Type": "application/json"},
        ).GET()

        if connect_request.status_code != 200:
            raise RequestError(
                "Error request, status code: {0}".format(connect_request.status_code)
            )

        return list[Channel](Channel(channel) for channel in connect_request.json())



from classes import Channel
from client import Client
from httpc import *


class ChannelOperations:

    """
    #### Channel operations...
    """

    def __init__(self, channel: Channel, client: Client):
        self.channel: Channel = channel
        self.client: Client = client

    def send_message(self, content: str) -> None:
        response = MentordRequest(
            url="https://discordapp.com/api/v9/channels/{0}/messages".format(
                self.channel.id
            ),
            headers={
                "Authorization": self.client.user.token,
                "Content-Type": "application/json",
            },
        ).POST(json={"content": content})

        if response.status_code != 200:
            raise Exception("Error request send...")



class ChannelIsNoneObject(Exception):
    ...


from .message import *
from .guild import *
from httpc import *


class Channel:

    """
    #### Channel object contains data
    #### about the channel in guild.
    """

    def __init__(self, channel: dict) -> None:
        self.channel: dict = channel

    def __check__(self) -> None:
        if self.channel is None:
            raise ChannelIsNoneObject("The Channel parameter was not passed...")

    @property
    def id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: id channel`
        """

        self.__check__()

        return int(self.channel["id"])

    @property
    def last_message_id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: last message id`
        """

        self.__check__()

        return int(self.channel["last_message_id"])

    @property
    def recipients(self) -> int:
        """
        Params:
            `None`

        Returns:
            `dict: recipients`

        Keys:
            `id: int (id member)`
            `username: str (username member)`
            `global_name: str (global_name member)`
            `avatar: str (avatar user hash)`
        """

        self.__check__()

        return dict(self.channel["recipients"])

    def messages(self, token: str) -> list[Message]:
        """
        Params:
            `None`

        Returns:
            `list[Message]: list messages object in current channel`
        """

        self.__check__()

        response = MentordRequest(
            url="https://discordapp.com/api/v9/channels/{0}/messages?limit=25".format(
                self.id
            ),
            headers={
                "Authorization": token,
                "Content-Type": "application/json",
            },
        ).GET()

        return list[Message](Message(message) for message in response.json())



class MemberIsNoneObject(Exception):
    ...

from classes import *
from client import Client
from events import *
from httpc import *
from utils import *

class Member:

    """
    #### Member object contains data
    #### about the participant.
    """

    def __init__(self, member: dict) -> None:
        self.member: dict = member

    def __check__(self) -> None:
        if self.member is None:
            raise MemberIsNoneObject("The member parameter was not passed...")

    @property
    def id(self) -> int:

        """
        Params:
            `None`

        Returns:
            `int: id member`
        """

        self.__check__()

        return int(self.member["id"])



class GuildIsNoneObject(Exception):
    ...

from classes import *
from client import Client
from events import *
from httpc import *
from utils import *

class Guild:

    """
    #### Guild object contains data
    #### about the guild.
    """

    def __init__(self, guild: dict) -> None:
        self.guild: dict = guild

    def __check__(self) -> None:
        if self.guild is None:
            raise GuildIsNoneObject("The guild parameter was not passed...")

    @property
    def id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: id guild`
        """

        self.__check__()

        return int(self.guild["id"])

    @property
    def name(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: name guild`
        """

        self.__check__()

        return str(self.guild["name"])

    @property
    def icon(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: icon guild in hash`
        """

        self.__check__()

        return str(self.guild["icon"])

    @property
    def owner_id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: owner guild id`
        """

        self.__check__()

        return int(self.guild["owner_guild"])



class RequestError(Exception):
    ...


import requests


class MentordRequest:

    """
    #### The class responsible for sending
    #### requests to the discord API.
    """

    def __init__(self, url: str, headers: dict) -> None:
        self.url: str = url
        self.headers: dict = headers

    def GET(self) -> requests.Response:
        """
        Params:
            `None`

        Returns:
            `requests.Response: output request`
        """

        return requests.get(url=self.url, headers=self.headers)

    def POST(self, json: dict = None) -> requests.Response:
        """
        Params:
            `dict: json data (Optional, default is None)`

        Returns:
            `requests.Response: output request`
        """

        if json is None:
            return requests.post(url=self.url, headers=self.headers)

        return requests.post(url=self.url, headers=self.headers, json=json)



class IncorrectToken(Exception):
    ...


from .classes import User
from .httpc import *
from .events import *


class Client:

    """
    #### Main client class.
    """

    def __init__(self) -> None:
        self.user: User = None

    def connect(self, token: str) -> None:
        """
        Params:
            `str`: token

        Returns:
            `None`
        """

        if token is None and token.__len__() <= 0:
            raise IncorrectToken("Check a token, it is invalid.")

        connect_request = MentordRequest(
            url="https://discordapp.com/api/v9/users/@me",
            headers={"Authorization": token, "Content-Type": "application/json"},
        ).GET()

        if connect_request.status_code != 200:
            raise IncorrectToken("ConnectionError: check a token, it is invalid.")

        self.user: User = User(connect_request.json(), token)



from threading import Thread
from time import sleep

from classes import User
from client import Client, IncorrectToken
from httpc import *

class on_ready:

    """
    #### The function will be called when the bot is fully ready.
    """

    def __init__(self, client: Client, rdelay: float = 5.0):
        self.client: client.Client = client
        self.rdelay: float = rdelay

    def __call__(self, func) -> None:
        while self.client.user is None:
            sleep(0.5)

        self.create_thread(self.reconnect_loop)

        func()

    def reconnect_loop(self) -> None:
        while True:
            connect_request = MentordRequest(
                url="https://discordapp.com/api/v9/users/@me",
                headers={
                    "Authorization": self.client.user.token,
                    "Content-Type": "application/json",
                },
            ).GET()

            if connect_request.status_code != 200:
                raise IncorrectToken(
                    "ConnectionError: check a token, it is invalid."
                )
                exit()

            self.client.user: User = User(
                connect_request.json(), self.client.user.token
            )

            sleep(self.rdelay)

    def create_thread(self, func) -> None:
        new_t = Thread(target=func)
        new_t.start()



from threading import Thread
from time import sleep

from classes import Message
from client import Client
from httpc import *
from requests import Response


class on_message:

    """
    #### The function will be called when a new message is received.
    """

    def __init__(self, client: Client, delay: float = 0.5):
        self.client: Client = client
        self.delay: float = delay

    def __call__(self, *func) -> None:
        while self.client.user is None:
            sleep(0.5)

        self.create_thread(func=self.update_loop, args=func)

    def get_messages_id(self) -> dict:
        messages: dict = {}

        for channel in self.client.user.get_channels():
            messages[channel.id] = channel.messages(token=self.client.user.token)[0].id

        return messages

    def get_request(self, channel_id: int, message_id: int) -> Response:
        return MentordRequest(
            url="https://discordapp.com/api/v9/channels/{0}/messages?after={1}&limit=1".format(
                channel_id, message_id
            ),
            headers={
                "Authorization": self.client.user.token,
                "Content-Type": "application/json",
            },
        ).GET()

    def update_loop(self, func) -> None:
        messages: dict = self.get_messages_id()

        while True:
            for channel in self.client.user.get_channels():
                try:
                    response = self.get_request(channel.id, messages[channel.id])
                except:
                    messages: dict = self.get_messages_id()

                if response.json().count != 0:
                    for message in response.json():
                        func(Message(message))
                        messages[channel.id] = Message(message).id

            sleep(self.delay)

    def create_thread(self, func, args) -> None:
        new_t = Thread(target=func, args=args)
        new_t.start()



from setuptools import setup

setup(
    name="mentord",
    version="1.1.8",
    description="A sync library for creating self bots in python!",
    packages=["mentord", "mentord/classes", "mentord/events", "mentord/utils"],
    long_description_content_type="text/markdown",
    long_description="""
### Example use:

```python
from mentord import *

User_client: Client = Client()
User_client.connect(token='YOUR TOKEN')
```

### Use decorators:

```python
@on_ready(client=User_client, delay=5.0)
def client_on_ready() -> None:
    print('Self-bot is started!')

@on_message(client=User_client, delay=0.5)
def client_on_message(message: Message) -> None:
    utils.channel_opers.ChannelOperations(channel=message.channel_id, client=User_client).send_message("hello!")
```""",
)



class MessageIsNoneObject(Exception):
    ...

from .channel import *
from .guild import *
from httpc import *
from .member import *


class Message:

    """
    #### Message object contains data
    #### about a message in a certain guild and in a certain channel.
    """

    def __init__(self, message: dict) -> None:
        self.message: dict = message

    def __check__(self) -> None:
        if self.message is None:
            raise MessageIsNoneObject("The message parameter was not passed...")

    @property
    def id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: id message`
        """

        self.__check__()

        return int(self.message["id"])

    @property
    def channel_id(self) -> int:
        """
        Params:
            `None`

        Returns:
            `int: channel id message`
        """

        self.__check__()

        return int(self.message["channel_id"])

    @property
    def content(self) -> str:
        """
        Params:
            `None`

        Returns:
            `str: content message`
        """

        self.__check__()

        return str(self.message["channel_id"])

    @property
    def author(self) -> str:
        """
        Params:
            `None`

        Returns:
            `User: author message`
        """

        self.__check__()

        return Member(self.message["author"])
